<?php

namespace Drupal\chart_js_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'html_inject_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "chart_js_field_formatter",
 *   label = @Translation("Chart.js Field Formatter"),
 *   field_types = {
 *     "chart_js_field_type"
 *   }
 * )
 */
class ChartJsFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $form['width'] = [
      '#title' => $this->t('Width'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('width'),
      '#description' => $this->t("Enter the width of the chart in pixels."),
    ];

    $form['height'] = [
      '#title' => $this->t('Height'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('height'),
      '#description' => $this->t("Enter the height of the chart in pixels."),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'width' => NULL,
      'height' => NULL,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = $this->t('Width: @width', ['@width' => $this->getSetting('width')]);
    $summary[] = $this->t('Height: @height', ['@height' => $this->getSetting('height')]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {

      // Get entity.
      $entity = $item->getEntity();

      // If the token module is enabled then do token replacement.
      $moduleHandler = \Drupal::service('module_handler');
      if ($moduleHandler->moduleExists('token')) {
        $token_data = [
          $entity->getEntityTypeId() => $entity,
        ];
        $data = \Drupal::token()->replace($item->data, $token_data, ['clear' => FALSE]);
        $options = \Drupal::token()->replace($item->options, $token_data, ['clear' => FALSE]);
      } else {
        $data = $item->data;
        $options = $item->options;
      }

      $config = \Drupal::config('chart_js_field.settings');

      // Uses the chart-js.thml.twig template.
      $element[$delta] = [
        '#theme' => 'chart_js',
        '#width' => $this->getSetting('width'),
        '#height' => $this->getSetting('height'),
        '#uid' => $entity->uuid(),
        '#type' => $item->type,
        '#data' => $data,
        '#options' => $options,
        '#external' => $config->get('external'),
      ];
    }

    return $element;
  }

}
